@@ -1,9 +1,12 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/DB/oracle.pm - oracle database backend
+# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
+# --
+# $Id: oracle.pm,v 1.62 2010/11/25 11:03:52 mg Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::DB::oracle;
@@ -11,12 +14,8 @@ package Kernel::System::DB::oracle;
 use strict;
 use warnings;
 
-our @ObjectDependencies = (
-    'Kernel::Config',
-    'Kernel::System::DateTime',
-    'Kernel::System::Log',
-    'Kernel::System::Main',
-);
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.62 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -32,22 +31,14 @@ sub LoadPreferences {
     my ( $Self, %Param ) = @_;
 
     # db settings
-    $Self->{'DB::Limit'}                       = 0;
-    $Self->{'DB::DirectBlob'}                  = 0;
-    $Self->{'DB::QuoteSingle'}                 = '\'';
-    $Self->{'DB::QuoteBack'}                   = 0;
-    $Self->{'DB::QuoteSemicolon'}              = '';
-    $Self->{'DB::QuoteUnderscoreStart'}        = '\\';
-    $Self->{'DB::QuoteUnderscoreEnd'}          = '';
-    $Self->{'DB::CaseSensitive'}               = 1;
-    $Self->{'DB::LikeEscapeString'}            = q{ESCAPE '\\'};
-    $Self->{'DB::MaxParamCountForInCondition'} = 1000;
-
-    # how to determine server version
-    $Self->{'DB::Version'}
-        = "SELECT CONCAT('Oracle ', version) FROM product_component_version WHERE product LIKE 'Oracle Database%'";
-
-    $Self->{'DB::ListTables'} = 'SELECT table_name FROM user_tables ORDER BY table_name';
+    $Self->{'DB::Limit'}            = 0;
+    $Self->{'DB::DirectBlob'}       = 0;
+    $Self->{'DB::QuoteSingle'}      = '\'';
+    $Self->{'DB::QuoteBack'}        = 0;
+    $Self->{'DB::QuoteSemicolon'}   = '';
+    $Self->{'DB::QuoteUnderscore'}  = '\\';
+    $Self->{'DB::CaseInsensitive'}  = 0;
+    $Self->{'DB::LikeEscapeString'} = q{ESCAPE '\\'};
 
     # dbi attributes
     $Self->{'DB::Attribute'} = {
@@ -64,10 +55,10 @@ sub LoadPreferences {
     # shell setting
     $Self->{'DB::Comment'}      = '-- ';
     $Self->{'DB::ShellCommit'}  = ';';
-    $Self->{'DB::ShellConnect'} = "SET DEFINE OFF;\nSET SQLBLANKLINES ON";    # must be on separate lines!
+    $Self->{'DB::ShellConnect'} = 'SET DEFINE OFF';
 
     # init sql setting on db connect
-    $Self->{'DB::Connect'} = "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'";
+    #$Self->{'DB::Connect'} = '';
 
     return 1;
 }
@@ -86,9 +77,8 @@ sub Quote {
             ${$Text} =~ s/;/$Self->{'DB::QuoteSemicolon'};/g;
         }
         if ( $Type && $Type eq 'Like' ) {
-            if ( $Self->{'DB::QuoteUnderscoreStart'} || $Self->{'DB::QuoteUnderscoreEnd'} ) {
-                ${$Text}
-                    =~ s/_/$Self->{'DB::QuoteUnderscoreStart'}_$Self->{'DB::QuoteUnderscoreEnd'}/g;
+            if ( $Self->{'DB::QuoteUnderscore'} ) {
+                ${$Text} =~ s/_/$Self->{'DB::QuoteUnderscore'}_/g;
             }
         }
     }
@@ -100,10 +90,7 @@ sub DatabaseCreate {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name!'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name!' );
         return;
     }
 
@@ -116,10 +103,7 @@ sub DatabaseDrop {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name!'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name!' );
         return;
     }
 
@@ -130,10 +114,6 @@ sub DatabaseDrop {
 sub TableCreate {
     my ( $Self, @Param ) = @_;
 
-    # get needed objects
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
-    my $MainObject   = $Kernel::OM->Get('Kernel::System::Main');
-
     my $SQLStart     = '';
     my $SQLEnd       = '';
     my $SQL          = '';
@@ -148,7 +128,6 @@ sub TableCreate {
     my $PrimaryKey   = '';
     my @Return       = ();
     my @Return2      = ();
-
     for my $Tag (@Param) {
 
         if (
@@ -156,7 +135,7 @@ sub TableCreate {
             && $Tag->{TagType} eq 'Start'
             )
         {
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $SQLStart .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQLStart .= $Self->{'DB::Comment'} . " create table $Tag->{Name}\n";
@@ -231,82 +210,69 @@ sub TableCreate {
         }
 
         # add primary key
+        my $Constraint = 'PK_' . $TableName;
+        if ( length $Constraint > 30 ) {
+            my $MD5 = $Self->{MainObject}->MD5sum(
+                String => $Constraint,
+            );
+            $Constraint = substr $Constraint, 0, 28;
+            $Constraint .= substr $MD5, 0,  1;
+            $Constraint .= substr $MD5, 31, 1;
+        }
         if ( $Tag->{PrimaryKey} && $Tag->{PrimaryKey} =~ /true/i ) {
-
-            my $Constraint = $Self->_ConstraintName(
-                TableName => $TableName,
+            push(
+                @Return2,
+                "ALTER TABLE $TableName ADD CONSTRAINT $Constraint"
+                    . " PRIMARY KEY ($Tag->{Name})"
             );
-
-            push @Return2,
-                "ALTER TABLE $TableName ADD CONSTRAINT $Constraint PRIMARY KEY ($Tag->{Name})";
         }
 
         # auto increment
         if ( $Tag->{AutoIncrement} && $Tag->{AutoIncrement} =~ /^true$/i ) {
-
-            my $Sequence = $Self->_SequenceName(
-                TableName => $TableName,
-            );
-
-            my $Trigger = $Sequence . '_t';
-
+            my $Sequence = 'SE_' . $TableName;
+            if ( length $Sequence > 28 ) {
+                my $MD5 = $Self->{MainObject}->MD5sum(
+                    String => $Sequence,
+                );
+                $Sequence = substr $Sequence, 0, 26;
+                $Sequence .= substr $MD5, 0,  1;
+                $Sequence .= substr $MD5, 31, 1;
+            }
             my $Shell = '';
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $Shell = "/\n--";
             }
-
-            push @Return2, <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'DROP SEQUENCE $Sequence';
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-            push @Return2, <<"EOF";
-CREATE SEQUENCE $Sequence
-INCREMENT BY 1
-START WITH 1
-NOMAXVALUE
-NOCYCLE
-CACHE 20
-ORDER
-EOF
-
-            push @Return2, <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'DROP TRIGGER $Trigger';
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-            push @Return2, <<"EOF";
-CREATE OR REPLACE TRIGGER $Trigger
-BEFORE INSERT ON $TableName
-FOR EACH ROW
-BEGIN
-    IF :new.$Tag->{Name} IS NULL THEN
-        SELECT $Sequence.nextval
-        INTO :new.$Tag->{Name}
-        FROM DUAL;
-    END IF;
-END;
-$Shell
-EOF
-
+            push( @Return2, "DROP SEQUENCE $Sequence" );
+            push( @Return2, "CREATE SEQUENCE $Sequence" );
+            push(
+                @Return2,
+                "CREATE OR REPLACE TRIGGER $Sequence"
+                    . "_t\n"
+                    . "before insert on $TableName\n"
+                    . "for each row\n"
+                    . "begin\n"
+                    . "  if :new.$Tag->{Name} IS NULL then\n"
+                    . "    select $Sequence.nextval\n"
+                    . "    into :new.$Tag->{Name}\n"
+                    . "    from dual;\n"
+                    . "  end if;\n"
+                    . "end;\n"
+                    . "$Shell",
+            );
         }
     }
 
-    # add unique
+    # add uniq
     for my $Name ( sort keys %Uniq ) {
-
-        my $Unique = $Self->_UniqueName(
-            Name => $Name,
-        );
-
+        my $Unique = $Name;
+        if ( length $Unique > 30 ) {
+            my $MD5 = $Self->{MainObject}->MD5sum(
+                String => $Unique,
+            );
+            $Unique = substr $Unique, 0, 28;
+            $Unique .= substr $MD5, 0,  1;
+            $Unique .= substr $MD5, 31, 1;
+        }
         if ($SQL) {
             $SQL .= ",\n";
         }
@@ -328,24 +294,34 @@ EOF
     for my $ForeignKey ( sort keys %Foreign ) {
         my @Array = @{ $Foreign{$ForeignKey} };
         for ( 0 .. $#Array ) {
-            push @{ $Self->{Post} },
+            push(
+                @{ $Self->{Post} },
                 $Self->ForeignKeyCreate(
-                LocalTableName   => $TableName,
-                Local            => $Array[$_]->{Local},
-                ForeignTableName => $ForeignKey,
-                Foreign          => $Array[$_]->{Foreign},
-                );
+                    LocalTableName   => $TableName,
+                    Local            => $Array[$_]->{Local},
+                    ForeignTableName => $ForeignKey,
+                    Foreign          => $Array[$_]->{Foreign},
+                ),
+            );
+
+            # generate forced index for every FK to do row locking (not table locking)
+            my $IndexName = $TableName . '_' . $Array[$_]->{Local};
+            if ( !$Index{$IndexName} ) {
+                $Index{ 'FK_' . $IndexName } = [ { Name => $Array[$_]->{Local} } ];
+            }
         }
     }
 
-    # add indexes
+    # add indexs
     for my $Name ( sort keys %Index ) {
-        push @Return,
+        push(
+            @Return,
             $Self->IndexCreate(
-            TableName => $TableName,
-            Name      => $Name,
-            Data      => $Index{$Name},
-            );
+                TableName => $TableName,
+                Name      => $Name,
+                Data      => $Index{$Name},
+            ),
+        );
     }
 
     return @Return;
@@ -354,13 +330,10 @@ EOF
 sub TableDrop {
     my ( $Self, @Param ) = @_;
 
-    # get config object
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
-
     my $SQL = '';
     for my $Tag (@Param) {
         if ( $Tag->{Tag} eq 'Table' && $Tag->{TagType} eq 'Start' ) {
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $SQL .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQL .= $Self->{'DB::Comment'} . " drop table $Tag->{Name}\n";
@@ -368,29 +341,8 @@ sub TableDrop {
                     . "----------------------------------------------------------\n";
             }
         }
-
         $SQL .= "DROP TABLE $Tag->{Name} CASCADE CONSTRAINTS";
-
-        # get sequence name
-        my $Sequence = $Self->_SequenceName(
-            TableName => $Tag->{Name},
-        );
-        my $Shell = '';
-        if ( $ConfigObject->Get('Database::ShellOutput') ) {
-            $Shell = "/\n--";
-        }
-
-        # build sql to drop sequence
-        my $DropSequenceSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'DROP SEQUENCE $Sequence';
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-        return ( $SQL, $DropSequenceSQL );
+        return ($SQL);
     }
     return ();
 }
@@ -398,9 +350,6 @@ EOF
 sub TableAlter {
     my ( $Self, @Param ) = @_;
 
-    # get config object
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
-
     my $SQLStart      = '';
     my @SQL           = ();
     my @Index         = ();
@@ -409,12 +358,11 @@ sub TableAlter {
     my $ReferenceName = '';
     my @Reference     = ();
     my $Table         = '';
-
     for my $Tag (@Param) {
 
         if ( $Tag->{Tag} eq 'TableAlter' && $Tag->{TagType} eq 'Start' ) {
             $Table = $Tag->{Name} || $Tag->{NameNew};
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $SQLStart .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQLStart .= $Self->{'DB::Comment'} . " alter table $Table\n";
@@ -425,123 +373,6 @@ sub TableAlter {
             # rename table
             if ( $Tag->{NameOld} && $Tag->{NameNew} ) {
                 push @SQL, $SQLStart . "ALTER TABLE $Tag->{NameOld} RENAME TO $Tag->{NameNew}";
-
-                my $Shell = '';
-                if ( $ConfigObject->Get('Database::ShellOutput') ) {
-                    $Shell = "/\n--";
-                }
-
-                # get old primary key name constraint
-                my $OldConstraint = $Self->_ConstraintName(
-                    TableName => $Tag->{NameOld},
-                );
-
-                # get new primary key name constraint
-                my $NewConstraint = $Self->_ConstraintName(
-                    TableName => $Tag->{NameNew},
-                );
-
-                # build SQL to rename primary key constraint
-                my $RenameConstraintSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'ALTER TABLE $Tag->{NameNew} RENAME CONSTRAINT $OldConstraint TO $NewConstraint';
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-                push @SQL, $SQLStart . $RenameConstraintSQL;
-
-                # build SQL to rename index of primary key
-                my $RenameIndexSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'ALTER INDEX $OldConstraint RENAME TO $NewConstraint';
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-                push @SQL, $SQLStart . $RenameIndexSQL;
-
-                # get old sequence name
-                my $OldSequence = $Self->_SequenceName(
-                    TableName => $Tag->{NameOld},
-                );
-
-                # get new sequence name
-                my $NewSequence = $Self->_SequenceName(
-                    TableName => $Tag->{NameNew},
-                );
-
-                # define old and new trigger names
-                my $OldTrigger = $OldSequence . '_t';
-                my $NewTrigger = $NewSequence . '_t';
-
-                # build SQL to rename sequence (only if a sequence exists)
-                my $RenameSequenceSQL = <<"EOF";
-DECLARE
-    sequence_count NUMBER;
-BEGIN
-    SELECT COUNT(*)
-    INTO sequence_count
-    FROM user_sequences
-    WHERE UPPER(sequence_name) = UPPER('$OldSequence');
-
-    IF sequence_count > 0 THEN
-        EXECUTE IMMEDIATE 'RENAME $OldSequence TO $NewSequence';
-    END IF;
-
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-                push @SQL, $SQLStart . $RenameSequenceSQL;
-
-                # build SQL to drop old trigger (only if a trigger exists)
-                # and to create a new trigger
-                # (the name of the autoincrement column needed for the trigger is investigated automatically)
-                my $DropOldAndCreateNewTriggerSQL = <<"EOF";
-DECLARE
-    trigger_count NUMBER;
-    pk_column_name VARCHAR(50);
-BEGIN
-    SELECT COUNT(*)
-    INTO trigger_count
-    FROM user_triggers
-    WHERE UPPER(trigger_name) = UPPER('$OldTrigger');
-
-    SELECT column_name
-    INTO pk_column_name
-    FROM user_ind_columns
-    WHERE UPPER(table_name) = UPPER('$Tag->{NameNew}')
-    AND UPPER(index_name) = UPPER('$NewConstraint');
-
-    IF trigger_count > 0 THEN
-        EXECUTE IMMEDIATE 'DROP TRIGGER $OldTrigger';
-
-        EXECUTE IMMEDIATE 'CREATE OR REPLACE TRIGGER $NewTrigger
-            BEFORE INSERT ON $Tag->{NameNew}
-            FOR EACH ROW
-            BEGIN
-                IF :new.' || pk_column_name || ' IS NULL THEN
-                    SELECT $NewSequence.nextval
-                    INTO :new.' || pk_column_name || '
-                    FROM DUAL;
-                END IF;
-            END;';
-    END IF;
-
-EXCEPTION
-    WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-                push @SQL, $SQLStart . $DropOldAndCreateNewTriggerSQL;
             }
             $SQLStart .= "ALTER TABLE $Table";
         }
@@ -616,33 +447,8 @@ EOF
                 $Default = defined $Tag->{Default} ? "'$Tag->{Default}'" : "''";
             }
 
-            # if the column is a CLOB we create a temporary column with the CLOB type,
-            # then copy the data from the old column to the temporary column
-            # and then remove the temporary column
-            # Fix/Workaround for ORA-22858: invalid alteration of datatype
-            if ( $Tag->{Type} eq 'CLOB' ) {
-
-                # create temp column name
-                my $ColumnTemp = $Tag->{Name} . '_TEMP';
-
-                # create temp column
-                push @SQL, "ALTER TABLE $Table ADD $ColumnTemp $Tag->{Type} NULL";
-
-                # copy data from old column into temp column
-                push @SQL, "UPDATE $Table SET $ColumnTemp = $Tag->{Name}";
-
-                # delete old column
-                push @SQL, "ALTER TABLE $Table DROP COLUMN $Tag->{Name}";
-
-                # rename temp column to old column name
-                push @SQL, "ALTER TABLE $Table RENAME COLUMN $ColumnTemp TO $Tag->{Name}";
-            }
-
-            # the column type is no CLOB
-            else {
-                my $SQLEnd = "ALTER TABLE $Table MODIFY $Tag->{Name} $Tag->{Type} DEFAULT NULL";
-                push @SQL, $SQLEnd;
-            }
+            my $SQLEnd = "ALTER TABLE $Table MODIFY $Tag->{Name} $Tag->{Type} DEFAULT NULL";
+            push @SQL, $SQLEnd;
 
             # handle default and require
             if ( $Default ne "''" && ( $Required || defined $Tag->{Default} ) ) {
@@ -709,7 +515,6 @@ EOF
             push @Reference, $Tag;
         }
     }
-
     return @SQL;
 }
 
@@ -719,43 +524,36 @@ sub IndexCreate {
     # check needed stuff
     for (qw(TableName Name Data)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
-    my $Index = $Self->_IndexName(
-        Name => $Param{Name},
-    );
-    my $CreateIndexSQL = "CREATE INDEX $Index ON $Param{TableName} (";
-    my @Array          = @{ $Param{Data} };
+    my $Index = $Param{Name};
+    if ( length $Index > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $Index,
+        );
+        $Index = substr $Index, 0, 28;
+        $Index .= substr $MD5, 0,  1;
+        $Index .= substr $MD5, 31, 1;
+    }
+    my $SQL   = "CREATE INDEX $Index ON $Param{TableName} (";
+    my @Array = @{ $Param{Data} };
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $CreateIndexSQL .= ', ';
+            $SQL .= ', ';
+        }
+        $SQL .= $Array[$_]->{Name};
+        if ( $Array[$_]->{Size} ) {
+
+            #           $SQL .= "($Array[$_]->{Size})";
         }
-        $CreateIndexSQL .= $Array[$_]->{Name};
     }
-    $CreateIndexSQL .= ')';
+    $SQL .= ')';
 
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
-    }
+    # return SQL
+    return ($SQL);
 
-    # build sql to create index within a "try/catch block"
-    # to prevent errors if index exists already
-    $CreateIndexSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE '$CreateIndexSQL';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-    return ($CreateIndexSQL);
 }
 
 sub IndexDrop {
@@ -764,37 +562,23 @@ sub IndexDrop {
     # check needed stuff
     for (qw(TableName Name)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!",
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
 
-    my $Index = $Self->_IndexName(
-        Name => $Param{Name},
-    );
-
-    my $DropIndexSQL = 'DROP INDEX ' . $Index;
-
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
+    my $Index = $Param{Name};
+    if ( length $Index > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $Index,
+        );
+        $Index = substr $Index, 0, 28;
+        $Index .= substr $MD5, 0,  1;
+        $Index .= substr $MD5, 31, 1;
     }
 
-    # build sql to drop index within a "try/catch block"
-    # to prevent errors if index does not exist
-    $DropIndexSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE '$DropIndexSQL';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-    return ($DropIndexSQL);
+    my $SQL = 'DROP INDEX ' . $Index;
+    return ($SQL);
 }
 
 sub ForeignKeyCreate {
@@ -803,52 +587,27 @@ sub ForeignKeyCreate {
     # check needed stuff
     for (qw(LocalTableName Local ForeignTableName Foreign)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
 
     # create foreign key name
-    my $ForeignKey = $Self->_ForeignKeyName(
-        Name => "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}",
-    );
-
-    # add foreign key
-    my $CreateForeignKeySQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
-        . "($Param{Local}) REFERENCES $Param{ForeignTableName} ($Param{Foreign})";
-
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
+    my $ForeignKey = "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}";
+    if ( length($ForeignKey) > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $ForeignKey,
+        );
+        $ForeignKey = substr $ForeignKey, 0, 28;
+        $ForeignKey .= substr $MD5, 0,  1;
+        $ForeignKey .= substr $MD5, 31, 1;
     }
 
-    # build sql to create foreign key within a "try/catch block"
-    # to prevent errors if foreign key exists already
-    $CreateForeignKeySQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE '$CreateForeignKeySQL';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
+    # add foreign key
+    my $SQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
+        . "($Param{Local}) REFERENCES $Param{ForeignTableName} ($Param{Foreign})";
 
-    # build index name
-    my $IndexName = $Self->_IndexName(
-        Name => 'FK_' . $Param{LocalTableName} . '_' . $Param{Local},
-    );
-
-    # generate forced index for every FK to do row locking (not table locking)
-    my @CreateIndexSQL = $Self->IndexCreate(
-        TableName => $Param{LocalTableName},
-        Name      => $IndexName,
-        Data      => [ { Name => $Param{Local} } ],
-    );
-
-    return ( $CreateForeignKeySQL, @CreateIndexSQL );
+    return ($SQL);
 }
 
 sub ForeignKeyDrop {
@@ -857,50 +616,26 @@ sub ForeignKeyDrop {
     # check needed stuff
     for (qw(LocalTableName Local ForeignTableName Foreign)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
 
     # create foreign key name
-    my $ForeignKey = $Self->_ForeignKeyName(
-        Name => "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}",
-    );
-
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
+    my $ForeignKey = "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}";
+    if ( length($ForeignKey) > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $ForeignKey,
+        );
+        $ForeignKey = substr $ForeignKey, 0, 28;
+        $ForeignKey .= substr $MD5, 0,  1;
+        $ForeignKey .= substr $MD5, 31, 1;
     }
 
     # drop foreign key
-    my $DropForeignKeySQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'ALTER TABLE $Param{LocalTableName} DROP CONSTRAINT $ForeignKey';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
+    my $SQL = "ALTER TABLE $Param{LocalTableName} DROP CONSTRAINT $ForeignKey";
 
-    # build index name
-    my $IndexName = $Self->_IndexName(
-        Name => 'FK_' . $Param{LocalTableName} . '_' . $Param{Local},
-    );
-
-    # build sql to drop index
-    my $DropIndexSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE 'DROP INDEX $IndexName';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-    return ( $DropForeignKeySQL, $DropIndexSQL );
+    return ($SQL);
 }
 
 sub UniqueCreate {
@@ -909,47 +644,36 @@ sub UniqueCreate {
     # check needed stuff
     for (qw(TableName Name Data)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
 
-    my $Unique = $Self->_UniqueName(
-        Name => $Param{Name},
-    );
+    my $Unique = $Param{Name};
+    if ( length $Unique > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $Unique,
+        );
+        $Unique = substr $Unique, 0, 28;
+        $Unique .= substr $MD5, 0,  1;
+        $Unique .= substr $MD5, 31, 1;
+    }
 
-    my $CreateUniqueSQL = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Unique UNIQUE (";
-    my @Array           = @{ $Param{Data} };
-    my $Name            = '';
+    my $SQL   = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Unique UNIQUE (";
+    my @Array = @{ $Param{Data} };
+    my $Name  = '';
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $CreateUniqueSQL .= ', ';
+            $SQL .= ', ';
         }
-        $CreateUniqueSQL .= $Array[$_]->{Name};
-        $Name            .= '_' . $Array[$_]->{Name};
+        $SQL  .= $Array[$_]->{Name};
+        $Name .= '_' . $Array[$_]->{Name};
     }
-    $CreateUniqueSQL .= ')';
+    $SQL .= ')';
 
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
-    }
+    # return SQL
+    return ($SQL);
 
-    # build SQL to create unique constraint within a "try/catch block"
-    # to prevent errors if unique constraint does already exist
-    $CreateUniqueSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE '$CreateUniqueSQL';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-    return ($CreateUniqueSQL);
 }
 
 sub UniqueDrop {
@@ -958,53 +682,34 @@ sub UniqueDrop {
     # check needed stuff
     for (qw(TableName Name)) {
         if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
             return;
         }
     }
 
-    my $Unique = $Self->_UniqueName(
-        Name => $Param{Name},
-    );
-
-    my $DropUniqueSQL = "ALTER TABLE $Param{TableName} DROP CONSTRAINT $Unique";
-
-    my $Shell = '';
-    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
-        $Shell = "/\n--";
+    my $Unique = $Param{Name};
+    if ( length $Unique > 30 ) {
+        my $MD5 = $Self->{MainObject}->MD5sum(
+            String => $Unique,
+        );
+        $Unique = substr $Unique, 0, 28;
+        $Unique .= substr $MD5, 0,  1;
+        $Unique .= substr $MD5, 31, 1;
     }
 
-    # build SQL to drop unique constraint within a "try/catch block"
-    # to prevent errors if unique constraint does not exist
-    $DropUniqueSQL = <<"EOF";
-BEGIN
-    EXECUTE IMMEDIATE '$DropUniqueSQL';
-EXCEPTION
-  WHEN OTHERS THEN NULL;
-END;
-$Shell
-EOF
-
-    return ($DropUniqueSQL);
+    my $SQL = "ALTER TABLE $Param{TableName} DROP CONSTRAINT $Unique";
+    return ($SQL);
 }
 
 sub Insert {
     my ( $Self, @Param ) = @_;
 
-    # get needed objects
-    my $ConfigObject   = $Kernel::OM->Get('Kernel::Config');
-    my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
-
     my $SQL    = '';
     my @Keys   = ();
     my @Values = ();
-    TAG:
     for my $Tag (@Param) {
         if ( $Tag->{Tag} eq 'Insert' && $Tag->{TagType} eq 'Start' ) {
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $SQL .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQL .= $Self->{'DB::Comment'} . " insert into table $Tag->{Table}\n";
@@ -1016,14 +721,15 @@ sub Insert {
         if ( $Tag->{Tag} eq 'Data' && $Tag->{TagType} eq 'Start' ) {
 
             # do not use auto increment values
-            next TAG if $Tag->{Type} && $Tag->{Type} =~ m{ ^AutoIncrement$ }xmsi;
-
+            if ( $Tag->{Type} && $Tag->{Type} =~ /^AutoIncrement$/i ) {
+                next;
+            }
             $Tag->{Key} = ${ $Self->Quote( \$Tag->{Key} ) };
             push @Keys, $Tag->{Key};
             my $Value;
             if ( defined $Tag->{Value} ) {
                 $Value = $Tag->{Value};
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
+                $Self->{LogObject}->Log(
                     Priority => 'error',
                     Message  => 'The content for inserts is not longer appreciated '
                         . 'attribut Value, use Content from now on! Reason: You can\'t '
@@ -1058,15 +764,18 @@ sub Insert {
             $Value .= ', ';
         }
         if ( $Tmp eq 'current_timestamp' ) {
-            if ( $ConfigObject->Get('Database::ShellOutput') ) {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $Value .= $Tmp;
             }
             else {
-                my $Timestamp = $DateTimeObject->ToString();
+                my $Timestamp = $Self->{TimeObject}->CurrentTimestamp();
                 $Value .= '\'' . $Timestamp . '\'';
             }
         }
         else {
+            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+                $Tmp =~ s/\n/\r/g;
+            }
             $Value .= $Tmp;
         }
     }
@@ -1105,129 +814,4 @@ sub _TypeTranslation {
     return $Tag;
 }
 
-sub _SequenceName {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    if ( !$Param{TableName} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need TableName!",
-        );
-        return;
-    }
-
-    my $Sequence = 'SE_' . $Param{TableName};
-    if ( length $Sequence > 28 ) {
-        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
-            String => $Sequence,
-        );
-        $Sequence = substr $Sequence, 0, 26;
-        $Sequence .= substr $MD5, 0,  1;
-        $Sequence .= substr $MD5, 31, 1;
-    }
-
-    return $Sequence;
-}
-
-sub _ConstraintName {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    if ( !$Param{TableName} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need TableName!",
-        );
-        return;
-    }
-
-    my $Constraint = 'PK_' . $Param{TableName};
-    if ( length $Constraint > 30 ) {
-        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
-            String => $Constraint,
-        );
-        $Constraint = substr $Constraint, 0, 28;
-        $Constraint .= substr $MD5, 0,  1;
-        $Constraint .= substr $MD5, 31, 1;
-    }
-
-    return $Constraint;
-}
-
-sub _UniqueName {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need Name!",
-        );
-        return;
-    }
-
-    my $Unique = $Param{Name};
-    if ( length $Unique > 30 ) {
-        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
-            String => $Unique,
-        );
-        $Unique = substr $Unique, 0, 28;
-        $Unique .= substr $MD5, 0,  1;
-        $Unique .= substr $MD5, 31, 1;
-    }
-
-    return $Unique;
-}
-
-sub _IndexName {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need Name!",
-        );
-        return;
-    }
-
-    my $Index = $Param{Name};
-    if ( length $Index > 30 ) {
-        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
-            String => $Index,
-        );
-        $Index = substr $Index, 0, 28;
-        $Index .= substr $MD5, 0,  1;
-        $Index .= substr $MD5, 31, 1;
-    }
-
-    return $Index;
-}
-
-sub _ForeignKeyName {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need Name!",
-        );
-        return;
-    }
-
-    my $ForeignKey = $Param{Name};
-    if ( length($ForeignKey) > 30 ) {
-        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
-            String => $ForeignKey,
-        );
-        $ForeignKey = substr $ForeignKey, 0, 28;
-        $ForeignKey .= substr $MD5, 0,  1;
-        $ForeignKey .= substr $MD5, 31, 1;
-    }
-
-    return $ForeignKey;
-}
-
 1;
