@@ -1,6 +1,9 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# bin/otrs.CreateTranslationFile.pl - create new translation file
+# Copyright (C) 2001-2011 OTRS AG, http://otrs.org/
+# --
+# $Id: otrs.CreateTranslationFile.pl,v 1.25 2011/01/12 08:54:08 mg Exp $
 # --
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU AFFERO General Public License as published by
@@ -9,26 +12,27 @@
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 # or see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 use strict;
 use warnings;
 
+# use ../ as lib location
 use File::Basename;
 use FindBin qw($RealBin);
 use lib dirname($RealBin);
-use lib dirname($RealBin) . '/Kernel/cpan-lib';
-use lib dirname($RealBin) . '/Custom';
+
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.25 $) [1];
 
 use Getopt::Std qw();
-
 use Kernel::Config;
 use Kernel::System::Encode;
 use Kernel::System::Log;
@@ -41,8 +45,8 @@ use Kernel::System::SysConfig;
 sub PrintUsage {
     print <<"EOF";
 
-otrs.CreateTranslationFile.pl - update translation files
-Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+otrs.CreateTranslationFile.pl <Revision $VERSION> - update translation files
+Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
 
 Translating OTRS
 ================
@@ -50,8 +54,8 @@ Translating OTRS
   Make sure that you have a clean system with a current configuration. No modules
   may be installed or linked into the system!
 
-    otrs.CreateTranslationFile.pl -l <Language>
-    otrs.CreateTranslationFile.pl -l all
+    otrs.CreateTranslationFile -l <Language>
+    otrs.CreateTranslationFile -l all
 
 Translating Extension Modules
 =============================
@@ -60,15 +64,8 @@ Translating Extension Modules
   that needs to be translated has to be installed or linked into the system, but
   only this one!
 
-    otrs.CreateTranslationFile.pl -l <Language> -m <module_directory>
-    otrs.CreateTranslationFile.pl -l all -m <module_directory>
-
-Optional Parameters
-=============================
-
-  To purge obsolete translation entries, use -p
-
-    otrs.CreateTranslationFile.pl -l all -p
+    otrs.CreateTranslationFile -l <Language> -m <module_directory>
+    otrs.CreateTranslationFile -l all -m <module_directory>
 
 EOF
 }
@@ -88,14 +85,12 @@ $CommonObject{SysConfigObject} = Kernel::System::SysConfig->new(%CommonObject);
 
 my $Home = $CommonObject{ConfigObject}->Get('Home');
 
-my $BreakLineAfterChars = 60;
-
 {
     my @Languages;
     my %Opts;
 
     # get opts
-    Getopt::Std::getopt( 'lmp', \%Opts );
+    Getopt::Std::getopt( 'lm', \%Opts );
 
     # check params
     if ( $Opts{l} && $Opts{l} eq 'all' ) {
@@ -119,47 +114,21 @@ my $BreakLineAfterChars = 60;
 
     print "Starting...\n\n";
 
-    # Gather some statistics
-    my %Stats;
-
     for my $Language (@Languages) {
         HandleLanguage(
-            Language      => $Language,
-            Module        => $Opts{m},
-            PurgeObsolete => exists $Opts{p} ? 1 : 0,
-            Stats         => \%Stats,
+            Language => $Language,
+            Module   => $Opts{m},
         );
     }
 
-    my %Summary;
-    for my $Language ( sort keys %Stats ) {
-        $Summary{$Language}->{Translated} = scalar grep {$_} values %{ $Stats{$Language} };
-        $Summary{$Language}->{Total} = scalar values %{ $Stats{$Language} };
-    }
-
-    print "\n\nTranslation statistics:\n";
-    for my $Language (
-        sort { $Summary{$b}->{Translated} <=> $Summary{$a}->{Translated} }
-        keys %Stats
-        )
-    {
-        my $Strings      = $Summary{$Language}->{Total};
-        my $Translations = $Summary{$Language}->{Translated};
-        print "\t" . sprintf( "%7s", $Language ) . ": ";
-        print sprintf( "%02d", int( ( $Translations / $Strings ) * 100 ) );
-        print sprintf( "%% (%4d/%4d)\n", $Translations, $Strings );
-
-    }
-
     print "\n\nFinished.\n";
 }
 
 sub HandleLanguage {
-    my (%Param) = @_;
+    my %Param = @_;
 
-    my $Language      = $Param{Language};
-    my $Module        = $Param{Module};
-    my $PurgeObsolete = $Param{PurgeObsolete};
+    my $Language = $Param{Language};
+    my $Module   = $Param{Module};
 
     my $ModuleDirectory = $Module;
     my $LanguageFile;
@@ -178,20 +147,12 @@ sub HandleLanguage {
         $Indent           = ' ' x 4;    # 4 spaces for module files
 
         # extract module name from module path
-        $Module = basename $Module;
+        $Module =~ s{ \A .* / (.+) /? \z }{$1}xms;
 
         # remove underscores and/or version numbers and following from module name
         # i.e. FAQ_2_0 or FAQ20
         $Module =~ s{ [_0-9]+ .+ \z }{}xms;
 
-        # special handling of some ITSM modules
-        if ( $Module eq 'ITSMIncidentProblemManagement' ) {
-            $Module = 'ITSMTicket';
-        }
-        elsif ( $Module eq 'ITSMConfigurationManagement' ) {
-            $Module = 'ITSMConfigItem';
-        }
-
         # save module directory in target file
         $TargetFile = "$ModuleDirectory/Kernel/Language/${Language}_$Module.pm";
     }
@@ -226,7 +187,8 @@ sub HandleLanguage {
     my $Data = '';
     my %UsedWords;
     my %UsedWordsMisc;
-    my $Directory = $IsSubTranslation
+    my $Directory
+        = $IsSubTranslation
         ? "$ModuleDirectory/Kernel/Output/HTML/$DefaultTheme"
         : "$Home/Kernel/Output/HTML/$DefaultTheme";
 
@@ -238,7 +200,7 @@ sub HandleLanguage {
     print "\nReading template files:\n";
 
     for my $File (@List) {
-        if ( open my $In, '<', $File ) {    ## no critic
+        if ( open my $In, '<', $File ) {
             my $Content = '';
             while ( my $Line = <$In> ) {
                 if ( $Line !~ /^#/ ) {
@@ -296,17 +258,8 @@ sub HandleLanguage {
                     $Translation =~ s/'/\\'/g;
                     my $Key = $Word;
                     $Key =~ s/'/\\'/g;
-
-                    $Param{Stats}->{$Param{Language}}->{$Word} = $Translation;
-
                     if ($Key !~ /(a href|\$(Text|Quote)\{")/i) {
-                        if (length($Key) < $BreakLineAfterChars) {
-                            $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
-                        }
-                        else {
-                            $Data .= $Indent . "\$Self->{Translation}->{'$Key'} =\n";
-                            $Data .= $Indent . '    ' . "'$Translation';\n";
-                        }
+                        $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
                     }
                 }
             }
@@ -321,19 +274,10 @@ sub HandleLanguage {
                     $UsedWords{$Word} = $LanguageCoreObject->{Translation}->{$Word};
                     my $Translation = $UsedWords{$Word} || '';
                     $Translation =~ s/'/\\'/g;
-
-                    $Param{Stats}->{$Param{Language}}->{$Word} = $Translation;
-
                     my $Key = $Word;
                     $Key =~ s/'/\\'/g;
                     if ($Key !~ /(a href|\$(Text|Quote)\{")/i) {
-                        if (length($Key) < $BreakLineAfterChars) {
-                            $Data .= $Indent . "'$Key' => '$Translation',\n";
-                        }
-                        else {
-                            $Data .= $Indent . "'$Key' =>\n";
-                            $Data .= $Indent . '    ' . "'$Translation',\n";
-                        }
+                        $Data .= $Indent . "'$Key' => '$Translation',\n";
                     }
                 }
             }
@@ -365,33 +309,19 @@ sub HandleLanguage {
         $UsedWordsMisc{$String} = 1;
 
         # lookup for existing translation
-        $UsedWords{$String} = ( $IsSubTranslation ? $LanguageObject : $LanguageCoreObject )->{Translation}
+        $UsedWords{$String}
+            = ( $IsSubTranslation ? $LanguageObject : $LanguageCoreObject )->{Translation}
             ->{$String};
 
         my $Translation = $UsedWords{$String} || '';
         $Translation =~ s/'/\\'/g;
-
-        $Param{Stats}->{ $Param{Language} }->{$String} = $Translation;
-
         my $Key = $String;
         $Key =~ s/'/\\'/g;
         if ($IsSubTranslation) {
-            if ( length($Key) < $BreakLineAfterChars ) {
-                $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
-            }
-            else {
-                $Data .= $Indent . "\$Self->{Translation}->{'$Key'} =\n";
-                $Data .= $Indent . '    ' . "'$Translation';\n";
-            }
+            $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
         }
         else {
-            if ( length($Key) < $BreakLineAfterChars ) {
-                $Data .= $Indent . "'$Key' => '$Translation',\n";
-            }
-            else {
-                $Data .= $Indent . "'$Key' =>\n";
-                $Data .= $Indent . '    ' . "'$Translation',\n";
-            }
+            $Data .= $Indent . "'$Key' => '$Translation',\n";
         }
     }
 
@@ -402,46 +332,32 @@ sub HandleLanguage {
     $Data .= $Indent . "# OBSOLETE ENTRIES FOR REFERENCE, DO NOT TRANSLATE!\n";
     $Data .= $Indent . "#\n";
 
-    if ( !$PurgeObsolete ) {
-        KEY:
-        for my $Key ( sort keys %{ $LanguageObject->{Translation} } ) {
+    KEY:
+    for my $Key ( sort keys %{ $LanguageObject->{Translation} } ) {
 
-            # ignore words from the core if we are translating a module
-            next KEY if $IsSubTranslation && exists $LanguageCoreObject->{Translation}->{$Key};
+        # ignore words from the core if we are translating a module
+        next KEY if $IsSubTranslation && exists $LanguageCoreObject->{Translation}->{$Key};
 
-            # ignore word if already used
-            next KEY if $UsedWordsMisc{$Key};
+        # ignore word if already used
+        next KEY if $UsedWordsMisc{$Key};
 
-            my $Translation = $LanguageObject->{Translation}->{$Key};
-            $Translation =~ s/'/\\'/g;
-            $Key =~ s/'/\\'/g;
+        my $Translation = $LanguageObject->{Translation}->{$Key};
+        $Translation =~ s/'/\\'/g;
+        $Key         =~ s/'/\\'/g;
 
-            # if a string was previously in a DTL, but has not yet been translated,
-            # there's no need to preserve it in the translation file.
-            next KEY if !$Translation;
+        # if a string was previously in a DTL, but has not yet been translated,
+        # there's no need to preserve it in the translation file.
+        next KEY if !$Translation;
 
-            # TODO: clarify if regular expression check is still needed
-            # in the past it was used to guard against wrong matches
-            next KEY if $Key =~ /(a href|\$(Text|Quote)\{")/i;
+        # TODO: clarify if regular expression check is still needed
+        # in the past it was used to guard against wrong matches
+        next KEY if $Key =~ /(a href|\$(Text|Quote)\{")/i;
 
-            if ($IsSubTranslation) {
-                if ( length($Key) < $BreakLineAfterChars ) {
-                    $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
-                }
-                else {
-                    $Data .= $Indent . "\$Self->{Translation}->{'$Key'} =\n";
-                    $Data .= $Indent . '    ' . "'$Translation';\n";
-                }
-            }
-            else {
-                if ( length($Key) < $BreakLineAfterChars ) {
-                    $Data .= $Indent . "'$Key' => '$Translation',\n";
-                }
-                else {
-                    $Data .= $Indent . "'$Key' =>\n";
-                    $Data .= $Indent . '    ' . "'$Translation',\n";
-                }
-            }
+        if ($IsSubTranslation) {
+            $Data .= $Indent . "\$Self->{Translation}->{'$Key'} = '$Translation';\n";
+        }
+        else {
+            $Data .= $Indent . "'$Key' => '$Translation',\n";
         }
     }
 
@@ -458,7 +374,9 @@ sub HandleLanguage {
         $NewOut = <<"EOF";
 $Separator
 # Kernel/Language/${Language}_$Module.pm - translation file
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2011 OTRS AG, http://otrs.org/
+$Separator
+# \$Id\$
 $Separator
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -468,7 +386,6 @@ $Separator
 package Kernel::Language::${Language}_$Module;
 
 use strict;
-use warnings;
 
 sub Data {
     my \$Self = shift;
@@ -481,7 +398,7 @@ EOF
 
     # translating the core
     else {
-        open( my $In, '<', $LanguageFile ) || die "Can't open: $LanguageFile\n";    ## no critic
+        open( my $In, '<', $LanguageFile ) || die "Can't open: $LanguageFile\n";
         while (<$In>) {
             if ( !$MetaData{DataPrinted} ) {
                 $NewOut .= $_;
@@ -489,7 +406,16 @@ EOF
             if ( $_ =~ /\$\$START\$\$/ && !$MetaData{DataPrinted} ) {
                 $MetaData{DataPrinted} = 1;
 
-                $NewOut .= "    # possible charsets\n";
+                my ( $Sec, $Min, $Hour, $Day, $Month, $Year )
+                    = $CommonObject{TimeObject}->SystemTime2Date(
+                    SystemTime => $CommonObject{TimeObject}->SystemTime(),
+                    );
+
+                $NewOut .= <<"EOF";
+    # Last translation file sync: $Year-$Month-$Day $Hour:$Min:$Sec
+
+    # possible charsets
+EOF
                 $NewOut .= "    \$Self->{Charset} = [";
                 for ( $LanguageCoreObject->GetPossibleCharsets() ) {
                     $NewOut .= "'$_', ";
@@ -510,7 +436,7 @@ EOF
 
                 if ( $LanguageCoreObject->{TextDirection} ) {
                     $NewOut .= <<"EOF";
-    # TextDirection rtl or ltr
+    # TextDirection rtl or ltr\n";
     \$Self->{TextDirection} = '$LanguageCoreObject->{TextDirection}';
 
 EOF
@@ -535,7 +461,7 @@ EOF
     }
 
     print "Writing $TargetFile\n";
-    open( my $Out, '>', $TargetFile ) || die $!;    ## no critic
+    open( my $Out, '>', $TargetFile ) || die $!;
     print $Out $NewOut;
     close $Out;
 }
